//===== 14.06.2020  CONTROLLER ===========
//==========  LIBRARIES CALL  =========================================
#include <Adafruit_GFX.h>
#include <gfxfont.h>
#include <Adafruit_PCD8544.h>
#include <OneWire.h>
#include <DallasTemperature.h>

#include <Wire.h>   // RTC LIBRARY

//===========  DS18B20 PIN DEFINITIONS  ===============================
#define ONE_WIRE_BUS 9

//===========  DS18B20 communication  =================================
char TempCelciusFahrenheit[6];
OneWire oneWire(ONE_WIRE_BUS);
DallasTemperature sensors(&oneWire);

//===========  NOKIA 5110 PIN DEFINITIONS  =============================
#define LCD_SCLK    6
#define LCD_DIN     5
#define LCD_DC      4
#define LCD_CS      3
#define LCD_RST     2
#define LCD_LIGHT   7
Adafruit_PCD8544 display = Adafruit_PCD8544(LCD_SCLK, LCD_DIN, LCD_DC, LCD_CS, LCD_RST);

////millis delay========================
int period = 1000;
unsigned long time_now = 0;

//===========  FAN PIN DEFINITIONS  ====================================
#define FAN_PWR    10

//===========  VARIABLES  ==============================================
float tempC = 0;
float tempF = 0;
float desired_temp = 25;
float deltaT = 0.2;
//===================RTC VARIABLES  ================================
const int DS1307 = 0x68; // Address of DS1307 see data sheets, SDA >> A4,   SCL >> A5
byte second = 0;
byte minute = 0;
byte hour = 0;

//======================================================================
void setup() {

  Wire.begin();              //
  Serial.begin(9600);        //
  printTime();               //
  
  
  //setuptime();               //    RTC starts with PREVIUS STTINGS, PERHAPS PUT THIS IN AN IF STATEMENT INSIDE LOOP WHEN BUTTON PRESSED SIMILARLY FOR PH...KAI TIMERS
   
  
  pinMode(LED_BUILTIN, OUTPUT);
  pinMode(LCD_LIGHT, OUTPUT);
  pinMode(FAN_PWR, OUTPUT);
  display.begin();
  display.setContrast(47);
  display.clearDisplay();
  sensors.begin();
  digitalWrite(LCD_LIGHT, HIGH);   // led on-off
}

//=====================================================================
void loop() {
  display.clearDisplay();
  WriteTime25110();
  WritePH25110();
   time_now = millis();
  printTime();                                 ////////////=========to com============
  while (millis() < time_now + period) {}
  sensors.requestTemperatures();
  tempC = sensors.getTempCByIndex(0);
  //  tempF = sensors.toFahrenheit(tempC);
  convertToString(tempC);
  WriteLCDmask ();
  checktemp ();
  display.display();
 }

//=================================================================
void convertToString(float number)
{
  dtostrf(number, 3, 1, TempCelciusFahrenheit);
}

//====================================================================
//==================================================================
void WriteLCDmask () {
  display.setCursor(0, 0);
  display.print(" (Te ) ");
  display.setCursor(0, 10);
  display.print(" (pH ) ");
  display.setCursor(71, 0);
  if (tempC < 0)  {
    display.println("");
  }
  else {
    display.setCursor(60, 40);
    display.print("C");
  }
  display.setCursor(60, 40);
  display.print("v.02");
}

//======================================================
void WriteTemp25110 ()
{ display.setCursor(40, 0);
  if (tempC < 0)  {
    display.println("*ERR*");
  }
  else {
    display.print(tempC);
  }
}

//=================================================


void WritePH25110 ()
{ // ============================need if statement here
  display.setCursor(40, 10);
  display.print("*ERR*");
}


void  WriteTime25110 () {
  char buffer[3];
  sprintf(buffer, "%02d", hour);
  display.setCursor(0, 40);
  display.print(" ");
  display.print(buffer);
  display.print(":");
  sprintf(buffer, "%02d", minute);
  display.print(buffer);
}
//==============   TEMP SECTION     ======================================================
void checktemp () {
  if (tempC > (desired_temp + deltaT))
  {
    analogWrite(FAN_PWR, 180); /// DIM FAN
    display.setCursor(26, 0);
    display.print("+");
    WriteTemp25110 ();
  }
  else if ((tempC < (desired_temp - deltaT)) && (tempC > 0) )
  {
    digitalWrite(FAN_PWR, LOW);
    display.setCursor(26, 0);
    display.print("-");
    WriteTemp25110 ();
  }
  else if (tempC < 0)
  {
    // display.println("      ");
    WriteTemp25110 ();
  }
  else {
    display.setCursor(26, 0);
    display.print(" ");
    WriteTemp25110 ();
  }

}

//=====================================================

byte decToBcd(byte val) {        // Convert normal decimal numbers to binary coded decimal
  return ((val / 10 * 16) + (val % 10));
}
byte bcdToDec(byte val) {        // Convert binary coded decimal to normal decimal numbers
  return ((val / 16 * 10) + (val % 16));
}

//==========TIME SECTION    =======this starts time setup ======================= coming from setup
void  setuptime() {

  Serial.println("Adjust time? Y/N");
  while (!Serial.available()) delay(10);
  if (Serial.read() == 'y' || Serial.read() == 'Y')   // This set of functions allows the user to change the date and time
  {
    Serial.read();
    setTime();
    Serial.print("Current Time: ");
  }
}


// keyboard input section   =================================         This set of codes is allows input of data

/// com functions ============================================

//======    input time by com ===========================
void setTime() {
  Serial.println();
  Serial.print("Hours, 0-24:  ");
  hour = readByte();
  Serial.println(hour);
  Serial.print("Minutes, 0-59:  ");
  minute = readByte();
  Serial.println(minute);
  second = 0;
  Serial.println("Done!");

  // The following codes transmits the data to the RTC ===========================================================
  Wire.beginTransmission(DS1307);
  Wire.write(byte(0));
  Wire.write(decToBcd(second));
  Wire.write(decToBcd(minute));
  Wire.write(decToBcd(hour));
  Wire.write(byte(0));
  Wire.endTransmission();     // Ends transmission of data
}

//=========================== works with the com  imput
byte readByte() {
  while (!Serial.available()) delay(10);
  byte reading = 0;
  byte incomingByte = Serial.read();
  while (incomingByte != '\n') {
    if (incomingByte >= '0' && incomingByte <= '9')
      reading = reading * 10 + (incomingByte - '0');
    else;
    incomingByte = Serial.read();
  }
  Serial.flush();
  return reading;
}

void readTime() {
  Wire.beginTransmission(DS1307);
  Wire.write(byte(0));
  Wire.endTransmission();
  Wire.requestFrom(DS1307, 7);
  second = bcdToDec(Wire.read());
  minute = bcdToDec(Wire.read());
  hour = bcdToDec(Wire.read());
}


//     =================   this is about COM output   ===========================
void printTime() {
  char buffer[3];
  readTime();
  Serial.print(" ");
  Serial.print(hour);
  Serial.print(":");
  sprintf(buffer, "%02d", minute);
  Serial.println(buffer);



}
